export var Random;
(function (Random) {
    function bool() {
        return Math.random() < 0.5;
    }
    Random.bool = bool;
    // randomize array in-place using Durstenfeld shuffle algorithm
    // ref: https://stackoverflow.com/a/12646864
    function shuffle(arr, inPlace = false) {
        arr = inPlace ? arr : [...arr];
        for (let i = arr.length - 1; i > 0; i--) {
            const j = Math.floor(Math.random() * (i + 1));
            [arr[i], arr[j]] = [arr[j], arr[i]];
        }
        return arr;
    }
    Random.shuffle = shuffle;
})(Random || (Random = {}));
export var RegularTable;
(function (RegularTable) {
    const TOP_LEVEL_TAGNAME = "TREE-FINDER";
    /*
     * click-related functions
     */
    function cellClicked(metadata) {
        return typeof (metadata === null || metadata === void 0 ? void 0 : metadata.y) !== "undefined" && !(metadata === null || metadata === void 0 ? void 0 : metadata.column_header_y);
    }
    RegularTable.cellClicked = cellClicked;
    function columnHeaderClicked(metadata) {
        var _a;
        return typeof ((_a = metadata) === null || _a === void 0 ? void 0 : _a.column_header_y) !== "undefined" && !!(metadata === null || metadata === void 0 ? void 0 : metadata.column_header);
    }
    RegularTable.columnHeaderClicked = columnHeaderClicked;
    function rowClicked(metadata) {
        return cellClicked(metadata) || rowHeaderClicked(metadata);
    }
    RegularTable.rowClicked = rowClicked;
    function rowHeaderClicked(metadata) {
        var _a;
        return typeof ((_a = metadata) === null || _a === void 0 ? void 0 : _a.row_header_x) !== "undefined" && !!(metadata === null || metadata === void 0 ? void 0 : metadata.row_header);
    }
    RegularTable.rowHeaderClicked = rowHeaderClicked;
    /*
     * debug utils
     */
    function clickLoggingListener(event, rt) {
        const metadata = metadataFromElement(event.target, rt, true);
        if (!metadata) {
            console.log(`event has no metadata`);
            return;
        }
        if (cellClicked(metadata)) {
            console.log(`cell clicked`);
        }
        if (columnHeaderClicked(metadata)) {
            console.log(`column header clicked`);
        }
        if (rowClicked(metadata)) {
            console.log(`row clicked`);
        }
        if (rowHeaderClicked(metadata)) {
            console.log(`row header clicked`);
        }
    }
    RegularTable.clickLoggingListener = clickLoggingListener;
    /*
     * general metadata-related functions
     */
    function metadataFromElement(target, rt, recursive = true) {
        if (target.tagName === TOP_LEVEL_TAGNAME) {
            return;
        }
        let metadata = rt.getMeta(target);
        if (!recursive || metadata || !target.parentElement) {
            return metadata;
        }
        return metadataFromElement(target.parentElement, rt, recursive);
    }
    RegularTable.metadataFromElement = metadataFromElement;
})(RegularTable || (RegularTable = {}));
export var Tag;
(function (Tag) {
    Tag.html = (strings, ...args) => strings
        .map((str, i) => [str, args[i]])
        .flat()
        .filter((a) => !!a)
        .join("")
        .replace(/>\s*\n\s*</g, '><')
        .replace(/\s*\n\s*/g, ' ');
})(Tag || (Tag = {}));
export var TreeHeader;
(function (TreeHeader) {
    const treeTemplate = document.createElement("template");
    function treeHeaderLevels({ isDir, isOpen, path }) {
        const tree_levels = path.slice(1).map(() => '<span class="rt-tree-group"></span>');
        if (isDir) {
            const group_icon = isOpen ? "remove" : "add";
            const tree_button = `<span class="rt-row-header-icon">${group_icon}</span>`;
            tree_levels.push(tree_button);
        }
        return tree_levels.join("");
    }
    function treeHeader({ isDir, isOpen, path }) {
        const tree_levels = treeHeaderLevels({ isDir, isOpen, path });
        const header_classes = !isDir ? "rt-group-name rt-group-leaf" : "rt-group-name";
        const header_text = path.length === 0 ? "TOTAL" : path[path.length - 1];
        treeTemplate.innerHTML = `<span class="rt-tree-container">${tree_levels}<span class="${header_classes}">${header_text}</span></span>`;
        return treeTemplate.content.firstChild;
    }
    TreeHeader.treeHeader = treeHeader;
})(TreeHeader || (TreeHeader = {}));
//# sourceMappingURL=util.js.map